/*howlong.c
 * part of RT version 2
 * Willard Goosey
 * goosey@nmt.edu
 * 95/11/14
 */

/* find out how much cpu time a child consumed.
 * run reset_child_time before forking and child_time after
 * to get cpu time (in minutes) of the child.
 *
 * calling child_time without ever calling reset_child_time gives the 
 * total time children have used.
 */
#include <sys/types.h>
#include <sys/times.h>

#define HZ 60 /*ticks per second */
#define NDEBUG

time_t child_time_offset = 0;

int reset_child_time()
{
   struct tms data;
   times(&data);
   child_time_offset = data.tms_cutime;
   return 0;
}

int child_time()
{
   struct tms data;
   time_t elapsed;

   times(&data);
   elapsed = data.tms_cutime - child_time_offset;
#ifdef DEBUG
   printf("elapsed time was %d\n",elapsed);
#endif
   elapsed = (elapsed / HZ) / 60; /*ticks -> minutes */
   return (int)elapsed + 1;
}
