/* Willard Goosey
 * exec_cmd.c
 * part of RT
 * cs328 fall 95
 */

#include "checkin.h"
#include <sys/file.h>
#define PATHLENGTH 80
#define NDEBUG
#define IN 0
#define OUT 1

int run_test (CASES *tests)
{
/*returns time taken to run test
 *expects stdin to be input
 */

/* stdin = 0
 * stdout = 1
 * stderr = 2
 */
   int testno,err,pid,c,yav;
   char sysret[80];
   int inpipe[2],outpipe[2],errpipe[2];

#ifdef DEBUG
   int bug;
#endif

   /*stdin needs to be put into CDIR/test-n.input */
   char newin[PATHLENGTH];
   char newout[PATHLENGTH];
   char newerr[PATHLENGTH];
   char newexit[PATHLENGTH];

/* build needed filenames */
   testno = getno(tests);
   sprintf(newin,"%s/test-%d.stdin",CDIR,testno );
   sprintf(newout,"%s/test-%d.stdout",CDIR,testno);
   sprintf(newerr,"%s/test-%d.stderr",CDIR,testno);

#ifdef DEBUG
   bug = dup(2);
#endif

   if((pipe(inpipe) == -1) || (pipe(outpipe)==-1)||(pipe(errpipe)==-1))
   {
      printf("couldn't open pipes\n");
      exit(10);
   }   
   if((pid=fork()) >0) /*parent*/
   {                   /* grabs input, tee's it to file and to inpipe*/
     int s_in,n;      
     char letter;

#ifdef DEBUG
     printf("first process\n");
#endif     

     if((s_in = open(newin,O_WRONLY | O_CREAT | O_TRUNC,0666)) == -1)
       exit(12);
     
     while((yav=read(0,&letter,1)) > 0)
     {
        if(write(s_in,&letter,yav) != 1)
          exit(5);
        if(write(inpipe[OUT],&letter,yav) != 1)
          exit(7);

     }  
     close(s_in);

/*done, so close all pipes to make children shut down */
     close(inpipe[IN]);
     close(inpipe[OUT]);
     close(outpipe[IN]);
     close(outpipe[OUT]);
     close(errpipe[IN]);
     close(errpipe[OUT]);

#ifdef DEBUG
     printf("first process waiting\n"); 
#endif
     wait(&err);
#ifdef DEBUG
     printf("first process done\n");
#endif
   }
   else if(pid == 0)  /*child*/
   {  
      int i,t,ppid;
      FILE *ecode;

#ifdef DEBUG
      printf("second process\n");
#endif

      if((ppid = fork()) >0) /* spawned a process*/
      {                      /* runs program */

/* close unneded pipe ends */
         close(inpipe[OUT]); 
         close(outpipe[IN]);
         close(errpipe[IN]);

/*make pipes be stdin, stdout, stderr */
         dup2(inpipe[IN],IN); /* redirect stdin */
         dup2(outpipe[OUT],OUT);/* stdout */
         dup2(errpipe[OUT],2);/* stderr*/

#ifdef DEBUG
         write(bug,"second process forking cli\n",28);
#endif


         reset_child_time();
         i = system(getcmd(tests));
         i = i >> 8; /* 0xNN00 is returned from system NN = exit code */
         t = child_time();

/* close all pipes so children won't halt on them */
         close(inpipe[IN]);
         close(outpipe[OUT]);
         close(errpipe[OUT]);
/*these are probably redundant.  So? */
         close(IN);
         close(OUT);
         close(2);

/* write exit code file */
         sprintf(newexit,"%s/test-%d.exit",CDIR,testno);
         ecode = fopen(newexit,"w");
         fprintf(ecode,"%d\n",i);
         fclose(ecode);

#ifdef DEBUG
         write(bug,"second process waiting",23);
#endif

         wait(0);
         /*return time the test took */
         exit(t);  /* yes, it's supposed to be an exit!*/
      }
      else if(ppid == 0)
      {  /* this process deals with stdout and stderr */
        if(fork() >0)
	{             /*this child does stdout */
           int out,letter,n;

#ifdef DEBUG
	   printf("third process\n");
#endif

           close(inpipe[IN]);
           close(inpipe[OUT]);
           close(outpipe[OUT]);
           close(errpipe[IN]);
           close(errpipe[OUT]);

           out = creat(newout,0666);
           while((n = read(outpipe[IN],&letter,1)) > 0)
	   {
             write(out,&letter,1);
             write(OUT,&letter,1);
           }

           close(out);
           close(outpipe[IN]);

#ifdef DEBUG
           printf("Third process waiting\n");

#endif

           wait(0);
#ifdef DEBUG
           printf("third process done\n");
#endif
           exit(0);
	}
        else
	{   /*this process does stderr */
           int out,letter,n;

#ifdef DEBUG
           write(bug,"fourth process\n",15);
#endif

           close(inpipe[IN]);
           close(inpipe[OUT]);
           close(outpipe[OUT]);
           close(outpipe[IN]);
           close(errpipe[OUT]);

           out = creat(newerr,0666);

           while((n = read(errpipe[IN],&letter,1)) > 0)
	   {
             write(out,&letter,1);
             write(2,&letter,1);
           }
           close(out);
           close(errpipe[IN]);
#ifdef DEBUG
           write(bug,"fourth process done\n",20);
#endif
           wait(0);
           exit(0);

        }
      }   
   }
   /*sh returns exit code of last command run
    *system returns exit code of sh */
   /* from man page of system:
    *   its termination status is contained in
    *   the 8 bits higher up from the low-order 8 bits of the  value
    *   returned by wait().
    */

   /* don't believe the hype!!!
    * returns time test took to run
    */
   err = err >> 8;
   return(err);
 }
      

