/****************************************************************
 * Author: Nathan  Paris
 * File: args.c
 * Purpose: parse command line for rt.
 *          rt [ -s | -v ] [ -f | nnn,nnn-nnn]
 *            -s silent, -v verbose
 *            -f failed cases, nnn, nnn-nnn subsets
 *****************************************************************/

#include "rt.h"

void inderr (char *);
int check_sv(char );
int check_f(void);
struct listtest *subset(char *);
void command_c( char );
int found(int,struct listtest *);
void swtch(int *, int *);
char *fail_str(void); 

extern int SILENT,
  FAILS,
  ALLTESTS;


/*************************************************************
 * Function: check_args
 * Purpose: checks command line options.
 *   If ALL test will be run test will point to NULL.
 *   Otherwise tests will be the head of the linked list of test
 *   to run.
 ************************************************************/


struct listtest *check_args (int argc, char *argv[])
{

  
  int str_cnt = 0,         /* where in individual string */
      arg_cnt = 1;         /* which argument */
  char c;
  struct listtest *Head;   /* linked list of test numbers */
  char *str;

  SILENT = NORMAL;
  FAILS = 0;

  while(arg_cnt < argc){
    
    if ( (c = argv[arg_cnt][str_cnt++]) == '-')
    {
      c = argv[arg_cnt][str_cnt];
      
      if( c == 'f' )
      {
	ALLTESTS = 0;
	FAILS = 1;
	Head = subset( fail_str() ); /* get string from failed file*/
	arg_cnt++;
	str_cnt=0;
      }
      else
      if(!check_sv(c))  /* if no command line errors  */
       {
	 command_c(c);		/* set appropriate flags */
	 arg_cnt++;
	 str_cnt=0;
       }
       
    }
    else
    if(isdigit(c))
    {
      if(!check_f()) /* cant have -f and digits */
      {
	ALLTESTS = 0;
        Head = subset(argv[arg_cnt]);
	
      }
      ++arg_cnt;
    }
    else
    {
     inderr("rt");
    }

  }  

  return(Head);  
}


/********************************************************
 * Function: check_sv
 * Purpose: checks rules for command line options
 *     for Silent and Verbose
 ********************************************************/
int check_sv(char c)
{
  if(c == 's')
  {
    if(SILENT == VERBOSE) /* if already been set */
      inderr("rt");     /* i.e. -v and -s used at same time*/
  } else 
  if (c == 'v')
  {
    if(SILENT == MUTE)
      inderr("rt");
  }
  return(0);
}

/********************************************************
 * Function: check_f
 * Purpose: checks rules for command line options
 *    for failed and subsets (options are mutually exclusive.
 ********************************************************/
int check_f(void)
{

  if(FAILS)    /* FAILS or subset not both */
  {
      inderr("rt");
  }
  return(0);
}

/***********************************************************
 * Function: fail_str()
 * Purpose: returns str from file "Rtresults/failed". the string
 *          is a list the failed cases of the last run.
 ***********************************************************/
char *fail_str()
{
  FILE *fp;
  char *buff,
       c;
  int cnt =0;

  if ( (fp = fopen(FAILED,"r")) == NULL )
  {
    fprintf (stderr,"Cannot access '%s'.\nAborting testing.\n",FAILED);
    exit(MISSCMD);
  }

 while ( (c = fgetc(fp)) != EOF)
     ++cnt;

  buff = (char * ) malloc( cnt * sizeof(char));
  rewind(fp);
  fscanf(fp,"%s",buff);
  fclose(fp);
  return(strdup(buff));
  
}

struct listtest *subset(char *set)
{
  struct listtest *new, *Head;
  char *ptr;
  char c,
     *buff,
      num[10];
  int size ,frst,limit,tmp,x;

  Head = (struct listtest *) malloc(sizeof(struct listtest));
  Head->next = NULL;
  
  buff = strdup(set);
  ptr =  strdup(set);
  
  while( (c = *ptr) != NULL)
  {
    size = 0;
    while( isdigit(c = *ptr)){
      ++ptr;
      size++;
    }
    strncpy(num,buff,size);
    sscanf(num,"%d",&tmp);
    for(x=0;x<10;++x)
      num[x]=NULL;
    buff += size;
    
    if(c == ',')
    {
      frst = tmp;
      limit = frst;
      ++ptr;			/* go past ',' */
      ++buff;                   /* past the ',' */
      if(!isdigit(*ptr))
	inderr("rt");
    }
    else
    if(c == '-')
    {
      frst = tmp;
      ++ptr;			/* go past '-' */
      ++buff;
      
      size = 0;
      
      if(!isdigit(*ptr))
	inderr("rt");
      
      while(isdigit(c = *ptr) ){
	++ptr;
	++size;;
      }
      strncpy(num,buff,size);
      buff += size;
      sscanf(num,"%d",&tmp);
      
      for(x=0;x<10;++x)      /* clear num */
	num[x]=NULL;
      limit =tmp;
    }
    else
    if(!isdigit(c) && c != NULL)
      inderr("rt");
    else{			/* last element */
      frst = tmp;
      limit = frst;
    }

    if(frst > limit)
      swtch(&frst,&limit);
    
    for(x=frst;x<=limit;++x)
    {
      if(!found(x,Head))
      {
	new = (struct listtest *) malloc (sizeof (struct listtest));
	new->next = Head->next;
	Head->next = new;
	new->tstnum = x;
      }
    }
 } /* while */
 return(Head);
}

/***************************************************************
 *
 *
 ***************************************************************/
void swtch(int *frst, int *limit)
{
  int tmp;

  tmp = *limit;
  *limit = *frst;
  *frst = tmp;
}

/************************************************
 * Function: found
 * Purpose: makes sure no duplicate tests
 ***********************************************/
int found(int num,struct listtest *Head )
{
  struct listtest *ptr;

  ptr = Head;
  while (ptr->next!= NULL)
  {ptr= ptr->next;
    if(ptr->tstnum == num)
      return(1);
  }
  return(0);
}

void rm_tst(int num,struct listtest *Head )
{
 struct listtest *ptr,*tmp;

  ptr = Head;
  while (ptr != NULL)
  {
    if( (ptr->next != NULL) && (ptr->next->tstnum == num))
    {
      ptr->next = ptr->next->next;
      fflush(stdout);
    }
    ptr=ptr->next;
  }
}


void inderr (char *cmd)
{
  fprintf(stderr,"Usage:  %s [-s | -v] [-f | nnn,nnn-nnn]\n",cmd);
  fprintf(stderr,"\t-s \truns in silent mode \n");
  fprintf(stderr,"\t-v \truns in verbose mode \n");
  fprintf(stderr,"\t-f \truns failed cases\n");
  fprintf(stderr,"\tnnn,nnn-nnn\n");
  fprintf(stderr,"\t \tsubsets of tests\n");
  exit (USAGE);
}

#ifdef DBUGMSG
/*found*/void debug_msg (char *msg)
{
  if (DEBUG)
    fprintf (stderr, "%s\n",msg);
}
#endif

void command_c( char c)
{
  switch ( c ){
    case 's':
      SILENT = MUTE;
      break;
    case 'v':
      SILENT = VERBOSE;
      break;
    case 'f':
      FAILS = 1;
      break;
    default:
      inderr("rt");
    }

  }
