/*
   Sean McCullough
   cs328
   regression tester
   ~jraloff/cs328/src
*/
/**********************************
**   Version 2                   **
**   Updated by Asa Shows        **
**********************************/
#include "rt.h"

int TotFailed=0;
extern char *GlobalDiff;
extern CASES *tests;

int prod_results (CASES *tests)
{
   int err;

   err=find_diffs(tests);
   compile_results(err, tests);
}

int diff_twofiles (char *diffstr, char *tofile, char *fromfile, char *header, char *outfile)
{
    int     ret;
    char    s[255];
    FILE*   f;

    /* Write out header */
    f=fopen (outfile, "at");
    if (f==NULL)
    {
	fprintf (stderr, "Unable to open diff file for diff output\n");
	return (-1);
    }

    fprintf (f, "%s", header);
    fclose (f);

    if (diffstr)
       sprintf(s,"diff %s %s %s >> %s\n",diffstr, tofile, fromfile, outfile);
    else
       sprintf(s,"diff %s %s >> %s\n",tofile, fromfile, outfile);
    
    /* shift out the wait stat, to get the actual return */
    ret = system(s)>>8; 
    /* if ret==1 then there were differences */
    if (ret==1)
    {
	return (1);
    }
    if (ret == 2)
    {
	fprintf(stderr, "Diff exited with 2 diffing %s %s\n", tofile, fromfile);
	return (-1);
    }
    return (0);
}

int find_diffs (CASES *tests)
{
   char tnum[17];
   char outfile[80]=RDIR, tnm[200], fnm[200];
   char bum[20];
   char *diffstr, *ofstr;
   int  differr=0, temp, n, i, filenum, last;
   FILE *f;

   /* Get test number into string tnum */
   n = getno(tests);
   sprintf(outfile,"%s/test-%d.diff",RDIR, n);

   /* Open diff file */
   f=fopen(outfile, "wt");

   if (f!=NULL)
   {
       /*Write test header*/
       fprintf(f,"::test %d: %s\n",n,getdesc(tests));

       /* If there was an error producing the test case */
       if (geterror(tests)!=0)
       {
	   switch (geterror(tests)) 
	   {
              case ERR_IN:
	       fprintf (f,"Missing input for test #%d\n",n);
	       break;
              case ERR_OUT:
	       fprintf(f,"Missing output for test #%d\n",n);
	       break;
	      case ERR_IO:
	       fprintf(f,"Missing input/output for test #%d\n",n);
	       break;
	      default:
	       break;
	   }
	   fclose(f);	   
	   return (1);
       }
       else
       {
	   fclose(f);
	
	   /*Setup diffstr*/
/*	   printf("GlobalDiff: %s\n",GlobalDiff); */
	   if (NULL==(diffstr=getdiffstr(tests)))
	      diffstr=GlobalDiff;
	   
	   /*First do diff of stdout file*/
	   sprintf (tnm,"%s/test-%d.stdout",RDIR,n);
	   sprintf (fnm,"%s/test-%d.stdout",CDIR,n);
	   temp=diff_twofiles (diffstr, tnm, fnm, "[stdout]\n", outfile);
	   if (temp==-1)
	      if (differr==0)
	         differr=temp;
	   if (temp==1)
	      differr=1;

	   /*Diff of stderr file*/
	   sprintf (tnm,"%s/test-%d.stderr",RDIR,n);
	   sprintf (fnm,"%s/test-%d.stderr",CDIR,n);
	   temp=diff_twofiles (diffstr, tnm, fnm, "[stderr]\n", outfile);
	   if (temp==-1)
	      if (differr==0)
	         differr=temp;
	   if (temp==1)
	      differr=1;

	   /*Now diff of exit code file*/
	   sprintf (tnm,"%s/test-%d.exit",RDIR,n);
	   sprintf (fnm,"%s/test-%d.exit",CDIR,n);
	   temp=diff_twofiles (diffstr, tnm, fnm, "[exit_code]\n", outfile);
	   if (temp==-1)
	      if (differr==0)
	         differr=temp;
	   if (temp==1)
	      differr=1;

	   /* Diff all output files */
	   if ((ofstr=getoutfiles (tests))!=NULL)
	   {
	       i=last=0;
	       filenum=0;
	       
	       do
	       {
		   /*Cycle through the string to next space or end of string*/
		   for (;i<strlen(ofstr);i++)
		      if (ofstr[i]==' ')
		         break;
		   
		   /*Setup filenames file*/
		   sprintf (tnm,"%s/test-%d.file%d", RDIR, n, filenum);
		   sprintf (fnm,"%s/test-%d.file%d", CDIR, n, filenum);
		   sprintf (bum,"[file%d]\n",filenum);
		   temp=diff_twofiles (diffstr, tnm, fnm, bum, outfile);
		   if (temp==-1)
		      if (differr==0)
		         differr=temp;
		   if (temp==1)
		      differr=1;
		   
		   /*Setup last */
		   last=i+1;
		   /*Increment over the space*/
		   i++;
		   /*Next file number*/
		   filenum++;
		   
	       }while (i<strlen(ofstr));
	   }
       }
   }
   else
   {
       fprintf (stderr, "Unable to open file for diff output %s", outfile);
       return (-1);
   }

   return differr;
}

void compile_results (int ferr, CASES* tests)
{
    char   s[257],strn[90], outfile[80]=RDIR;
    int    n, outlen;
    static int first=0, second=0;
    FILE   *f;

    /* Get test number into string tnum */
    n = getno(tests);
    sprintf(outfile,"%s/test-%d",RDIR, n);
    outlen=strlen (outfile);
    strcpy (&(outfile[outlen]), ".diff");

    /* If tests failed */
    if (ferr==1)
    {
      if (first==0)
      {
	sprintf(s,"cat %s > %s\n",outfile,RESULTS);
	first=1;
      }
      else
      {
	sprintf(s,"cat %s >> %s\n",outfile,RESULTS);
      }
	system(s);
	sprintf(s,"rm -f %s\n",outfile);
	system(s);
    }

    /* If no tests failed */
    if (ferr==0)
    {
	strcpy (&(outfile[outlen]), ".*");
	sprintf(s,"rm -f %s\n",outfile);
	system(s);
    }
    
    /* If it is equal to -1 don't know what is going on so just leave stuff 
     * AS IS */

    /* Update failed file */
    if (ferr==1 || ferr==-1)
    {
	TotFailed++;
	
	f=fopen (FAILED,"at");
	if (f!=NULL)
	{
	    /* first time running this one */
	    if (second==0)
	    {
		fprintf (f, "%d", n);
		second=1;
	    }
	    else
	        fprintf (f, ",%d", n);
	    fclose (f);
	}
	else
	   fprintf (stderr, "Unable to add failed test %d to failed file\n", n);
    }

}

void PrintTotalFailed ()
{
    if (SILENT==NORMAL || SILENT==VERBOSE)
    {
	printf ("%d out of %d tests failed.\n",TotFailed,tests->amount);
    }
}








