/* ====================================================== */
/*                                                        */
/*                  Software Construction                 */
/*                         CS 328                         */
/*                       Fall 1995                        */
/*               Regression Tester Version 1              */
/*                           rt                           */
/*                                                        */
/*                          team:                         */
/*               Manuel Gallegos -- reality               */
/*                Willard Goosey -- goosey                */
/*               Sean McCullough -- bankshot              */
/*               Jonathan Raloff -- jraloff               */
/*                                                        */
/* ====================================================== */
/* ====================================================== */
/*                                                        */
/*                  Software Construction                 */
/*                         CS 328                         */
/*                       Fall 1995                        */
/*               Regression Tester Version 2              */
/*                    rt and rtresults                    */
/*                                                        */
/*                          team:                         */
/*          Nathan Paris  nparis@cs     835-9638    (N)   */
/*             Asa Shows  showsa@nmt    835-6293    (A)   */
/*         Randall Jones  rjones@cs     835-2917    (Ra)  */
/*             Bill Crum  tobor@cs      835-6060    (T)   */
/*        Jason Brackins  jbrackin@nmt  835-1813    (J)   */
/*                                                        */
/* ====================================================== */


/* 
    (J) main                                  
          |
    (N)   |--- check_args    -- add for -f take care of subset communication
    (N)   |--- flush_results 
          |      |
          |      +--- file_exist
          |
    (N)   |--- parse_cfile -- take care of subsets get struct from check_args
          |      |
    (N)   |      |--- createlist 
    (N)   |      |--- newcase 
    (N)   |      |--- setcmd 
    (N)   |      |--- settno 
    (N)   |      +--- setdesc 
          |
    (Ra)  |--- exec_tests -timeout 
          |     |
    (Ra)  |     +--- run_test 
          |          |
    (Ra)  |          |--- getcmd
    (Ra)  |          |--- seterror
    (Ra)  |	     |--- exec_cmd
          |          |      |
    (Ra)  |          |      |--- getcmd
    (Ra)  |          |      +--- getno
          |          |
    (Ra)  |	     +--- create_output
          |          |       |
    (Ra)  |          |       +--- gettno
          |          |
    (A)   |          +--- prod_results
	  |	     |
    (A)   |          |--- find_diffs
          |          |      |
    (A)   |          |      |--- getno
    (A)   |          |      |--- getdesc
    (A)   |          |      +--- geterror
	  |          |      
    (A)   |          +--- compile_results
          |
    (J)   *-- file_exist
    (R)   *-- nextcase 
    (R)   *-- firstcase 
    (R)   *-- lastcase

    (T)   WHY                                                  */

/*   rt.h   */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int DEBUG;                      /* boolean for debugging */

int SILENT;                     /* boolean for silent/verbose reporting */
#define     NORMAL        0     /* defines for int SILENT; */
#define     VERBOSE       1
#define     MUTE          -1

/* extern CASES *tests; */
int yylval;
char readch;

/* *********** defn in lex.yy.c from scan.lex *********** */
extern int yyleng;
extern char yytext[];
extern FILE *yyin;
/* extern int GlobalTimeout; */
/* extern char *GlobalDiff; */

struct casestruct {                   /* instance of each case */
    char *diffstr;
    int  timeout;
    char *outfiles;
    char *cmdln;
    int testno;
    char *desc;
    int ERROR;
};

struct teststruct {                   /* instance of each test */
       struct casestruct *caseptr;
       struct teststruct *next;
       } *first_test;

typedef struct liststruct {           /* defn of class CASES */
       struct teststruct *head;
       struct teststruct *curr;
       int amount;
       } CASES;

struct listtest {
  struct listtest *next;  /* nparis  */
  int tstnum;
};


/* ********** filename constructs ********** */
#define   RDIR           "RTresults"           /* location of results dir */
#define   CDIR           "RTtests"             /* location of cases dir */
#define   TMP_OUT        "RTresults/stdout"    /* filename for temp stdout */
#define   TMP_ERR        "RTresults/stderr"    /* filename for temp stderr */
#define   RESULTS        "RTresults/results"   /* filename for results file */
#define   CMDF           "RTtests/commands"    /* filename for cmd file */

#define   FAILED         "RTresults/failed"    /* filename for failed file */

#define TEST_LBL      "test-"      /* test filename prefix */
#define INPUT_EXT     ".stdin"     /* input file postfix */
#define STDOUT_EXT    ".stdout"    /* stdout file postfix */
#define STDERR_EXT    ".stderr"    /* stderr file postfix */
#define EXITCODE_EXT  ".exit"      /* exit code file postfix */

/* ********** time constants ********** */
#define SEC_PER_MIN   60
#define SHMEM_ERR    -47

/* ********** exit codes ********** */
#define   USAGE         -1              /*  incorrect options usage  */
#define   MISSCMD       -2              /*  CMDF missing  */
#define   DUPTEST       -3              /*  duplicate tests in CMDF  */
#define   NOTEST        -4              /*  no tests parsed from CMDF  */
#define   NOWRITE       -5              /*  unable to write output  */
     /* all other non-zero exit codes 
      * (codes > 0) indicate # tests failed 
      * exit code of 0 indicates success
      */

/* ********** error codes  ********** */
#define   ERR_IN         42            /* CDIR input missing */
#define   ERR_OUT        54            /* CDIR output missing */
#define   ERR_IO         72            /* CDIR input&output missing */
/* no longer checking if exec file exists
#define   ERR_EXEC       72             cmd[0] missing */



/* ********** debug message ********** */
#define DBG_MSG_LENGTH  128
#ifndef DEBUG_MESSAGE
#define DEBUG_MESSAGE DBG_MSG_LENGTH
char dbgMsg[DBG_MSG_LENGTH];  /* char string area for debug/error messages */
#else
extern char dbgMsg[];
#endif


int prod_results (CASES *tests);      
    /* returns the # of failed cases 
     * takes the object tests of class CASES
     * calls find_diffs
     * counts the # of tests ran
     * adds the # of unran tests (file missing)
     * calls compile_results
     */
int find_diffs (CASES *tests);                    
    /* returns the # of failed cases
     * takes the object tests of class CASES
     * finds the differences between the created output files
     * with the previously existing output files and creates a
     * diff file for the cases that produce differing outputs
     * removes the RDIR output files if no differences
     */
void compile_results (int ferr, CASES *tests);
    /* takes the total # of tests run and the # of tests failed
     * takes all diff files and compiles RESULTS
     * produces summary info -- x of y passed
     * checks SILENT to display to stdout or not
     */


/* ==================== flush.c ==================== */
void flush_results ( );
    /* deletes the contents of RDIR & creates if it does not exist
     */

/* ==================== exec_cmd.c ==================== */
int exec_cmd (CASES *tests);                      
    /* returns the command line's exit code, -1 on error
     * forks and execs program being tested
     * file output:
     *        TMP_OUT & TMP_ERR
     * /

/* ==================== exec_tests.c ==================== */
void exec_tests (CASES *tests);
    /* takes the object tests of the class CASES
     * runs a loop for all tests checking if all needed
     * files exist -- CDIR inputs & outputs, & the exec file
     * if any missing, sets an error flag in CASES tests
     * else calls run_test
     */

/* ==================== run_test.c ==================== */
void run_test (CASES *tests);
    /* takes the object tests of the class CASES
     * calls exec_cmd 
     * calls create_output
     */

/* ==================== create_output.c ==================== */
void create_output (int testnum);
    /* takes exit_code of the ran program and the test #
     * compiles the files exec_cmd created
     *           --- TMP_OUT & TMP_ERR
     * with exit_code creating the file RDIR output
     */

void PrintTotalFailed ();
    /* Prints total failed */

/* ==================== file_exist.c ==================== */
int file_exist (char *file_name);
    /* returns 0,1,2,3 
     * takes a string file_name
     * checks if file_name exists 
     * returns:
     *         1 --  executable file
     *         2 --  readable file
     *         3 --  readable/writable directory
     */

/* ==================== args.c ==================== */
struct listtest *check_args (int argc, char *argv[]);
    /* parses the command line options
     * sets SILENT if -s specified
     * sets FAILED if -f specified
     * sets the pointer "tests" to beggining
     * of linked lists of tests
     * If ALL tests chould be ran: test = NULL
     */
#ifdef DBUGMSG
/*found*/void debug_msg (char *msg);
    /* takes in a character string for a message that
     * could be used in debugging (if DEBUG set)
     */
#endif
int found(int,struct listtest *);
   /* checks if tst number is in linked list */
void rm_tst(int,struct listtest *);

/* ==================== pcfile.c ==================== */
void parse_cfile (struct listtest *);
    /* takes the test number to run
     * uses file_exist to see if CMDF exists
     * and exits with MISSCMD if it does not
     * else it scans through CMDF getting the desired info
     * if no tests found, exits with NOTEST
     */

/* ==================== cases.c ==================== */
int geterror (CASES *tests);
    /* returns the error for the current case
     * takes the instance tests of the class CASES
     */
char *getcmd (CASES *tests);
    /* returns the command line for the current case
     * takes the instance tests of the class CASES
     */
int getno (CASES *tests);
    /* returns the test number for the current case
     * takes the instance tests of the class CASES
     */
char *getdesc (CASES *tests);
    /* returns the description for the current case
     * takes the instance tests of the class CASES
     */
char *getoutfiles (CASES *tests);
    /* returns the list of output files for the current
     * case Null if none
     */
char *getdiffstr (CASES *tests);
    /* returns the diff string if one exists otherwise returns
     * null
     */
void nextcase (CASES *tests);
    /* takes the instance tests of the class CASES
     * moves to the next case 
     */
void firstcase (CASES *tests);
    /* takes the instance tests of the class CASES
     * moves to the first case listed in the instance
     */
int lastcase (CASES *tests);
    /* returns 1 if at last instance of CASES, 0 otherwise
     * takes the instances tests of the class CASES
     */
CASES *createlist ( );
    /* returns an instance of CASES
     * creates an instance of CASES 
     */
void matchtest (CASES *tests, int newtno);
    /* takes the instance of tests of the class CASES & new test to add
     * exits with DUPTEST if newtno found in tests
     */
void newcase (CASES *tests, int newtno);
    /* takes the instance tests of the class CASES
     * creates a new case for the object tests
     */
void setcmd (CASES *tests, char *cmd);
    /* takes the instance tests of the class CASES and the command line
     * places cmd into the current instance of CASES
     */
void settno (CASES *tests, int tno);
    /* takes the instance tests of the class CASES and the test number
     * places tno into the current instance of CASES
     */
void setdesc (CASES *tests, char *desc);
    /* takes the instance tests of the class CASES and the description
     * places desc into the current instance of CASES
     */
void seterror (CASES *tests, int error);
    /* takes the instance tests of the class CASES and the error
     * places error into the current instance of CASES
     */
void setdiff (CASES *, char *); /*nparis */
void setoutf (CASES *, char *); /*nparis */
void settimeout (CASES *, int); /*nparis */


